{% load i18n query_string_tools %}
{% if page.has_previous %}<a href="{{ request.path }}{% query_string request.GET page=page.previous_page_number %}">{% trans "Previous" %}</a>{% endif %}
{% for num in page.page_range %}
    {% if not num %}
    ...
    {% else %}
    <a href="{{ request.path }}{% query_string request.GET page=num %}">{{ num }}</a>
    {% endif %}
{% endfor %}
{% if page.has_next %}<a href="{{ request.path }}{% query_string request.GET page=page.next_page_number %}">{% trans "Next" %}</a>{% endif %}
